#!/usr/bin/env python
##\file StartManyJobs.py
##\brief Start ManyJobs.
##\author Hideki Fujioka
##\date 11/09/2009
"""
\mainpage ManyJobs

\section intro_sec Introduction

This tool submits jobs to many clusters remotely 
from one machine (maybe your desktop).  
When a job starts, the computing node contacts 
to the master-host (maybe your desktop) and 
the master-host sends parameters for a job and 
set in environmental variables.  
When a job ends, the computing node contacts to 
the master-host (maybe your desktop) and the 
master-host submit a new job.  
One can set job dependency; if job A depends on 
job B, job A won't be assigned until job B ends.
Currently, all communication is done with 'ssh' 
command. 

It is necessary to set up 'ssh' to work to / from 
he clusters without password.
ManyJobs is a python based tool for managing ManyTasks 
on widely distributed supercomputing resources.
To date it has been used in production mode on the following machines:
Abe\@NCSA,
Lonestar\@TACC,
Ranger\@TACC,
QueenBee\@LONI,
Kraken\@NICS,
Steele\@RCAC,
(eric,louie,poseidon,painter,oliver)\@LONI,
and the CCS Cluster at Tulane's CCS.
All you need is an account with a user allocation 
in one of the above machines and you too can run Many Jobs.

\section step1 Requirements
\li ManyJobs is written in Python (>=2.4) script only.\n
\li You have to set ssh authentication to ssh/gsissh to/from remote cluster head nodes without password by use of password keys.

\section step2 Getting Started
\li Check python installation \n
\li Setting SSH/GSISSH password keys for both directions \n
\li Prepare \ref DataBaseFile. \n
\li Prepare \ref JobSubmisionScripts. \n
\li Prepare User Tasks Scripts. \n
\li Setting User executable files on each cluster. \n

\section user_command User Commands
\li StartManyJobs.py
\li DeleteAll.py
\li Flush.py
\li ClearDataBase.py
\li CopyScript.py

\section acknowledgement Acknowledgement
ManyJob is developed as part of a LONI Institute Project to Tom Bishop,
Shantenu Jha and Hideki Fujioka. Fujioka is the main developer. ManyJob
provides a simple self-contained implementation of the SAGA-based BigJob
concept. ManyJob has received further support from NSF Cybertools and
LaSIGMA Projects.
\n
-----------------------------------------------------------------------

\page StartManyJobs Start ManyJobs

StartManyJobs.py is the command to initiate ManyJobs.

Usage: StartManyJobs.py [\ref DataBaseFile]\n 

For the details of DataBase file, see \ref DataBaseFile.\n





"""
#
#
# 06/01/2011 Last Modified by Hideki Fujioka 
#
import os
import sys
import string
import shutil
#
#
if (len(sys.argv) < 2):
    print "%s [ManyJOBS Data Base File\n" % (sys.argv[0])
    sys.exit()
if (False == os.path.isfile(sys.argv[1])):
    sys.exit("DB not exist")
#
# get ManyJOBS dir
ManyJobsDir = os.path.dirname(os.path.abspath(sys.argv[0])) + "/localbin"
#
# Set Path and Import Modules
sys.path.append(ManyJobsDir)
import ManyJobsManager
import PauseList
import StatusDatabase
from submit_job import *
#
#
# get Hosts & Jobs Data Base File name
manyJobsDB = os.path.abspath(sys.argv[1])
#
# Copy XSLT file to working dir
shutil.copyfile(ManyJobsDir+"/MJstatus.xslt",
                os.path.dirname(manyJobsDB)+"/MJstatus.xslt")
#
# Initialize Status Database
statusDB = StatusDatabase.StatusDatabase(manyJobsDB)
if (statusDB.File == 1):
    print "Detele ", statusDB.StatusDatabase," Before Submit\n"
    statusDB.finalize()
    sys.exit()
statusDB.finalize()
#
# Initialize Paused Hosts List
pList = PauseList.PauseList(manyJobsDB)
pList.initialize()
pList.finalize()
#
#
#
msg = "********** START MANY JOBS ***********"
log_file(os.path.dirname(manyJobsDB),msg)
msg = "Data Base File : " + sys.argv[1]
log_file(os.path.dirname(manyJobsDB),msg)
msg = "**************************************"
log_file(os.path.dirname(manyJobsDB),msg)
#
#
# init ManyJOBS manager
mjm = ManyJobsManager.ManyJobsManager(manyJobsDB)
#
#
# Get Local Host
UserHostName = mjm.getLocalHost()
#
# get host information
name = []
rhost = []
maxJobNum = []
jobLanch = []
[name,rhost,jobLanch,maxJobNum] = mjm.getClusterList()
#
# get secure copy
userscp = mjm.getSecureCopy()
#
# get secure shell
userssh = mjm.getSecureShell()
#
# get option string
optionString = []
for na in name:
    optionString.append(mjm.getOptionStringByName(na))    
#
# finalize ManyJOBS manager
mjm.finalize()
#
#
# Submit JOBS
for i in range(len(name)):
    # Create LOG dirctory on Remote Host
    remoteUsrPath = os.path.dirname(jobLanch[i])
    remoteLogPath = os.path.dirname(remoteUsrPath)+"/log"
    command = "%s %s mkdir -p %s" % (userssh,rhost[i],remoteLogPath)
    commands.getoutput(command)
    #
    for j in range(int(maxJobNum[i])):
        #
        ## Submit Job
        submitJob(name[i],rhost[i],jobLanch[i],UserHostName,ManyJobsDir,manyJobsDB,optionString[i])
	time.sleep(1)
#

