#
#
# Paused Host List
#
# 08/11/2010 Hideki Fujioka
# 03/26/2012 Hideki Fujioka Modifiy
#
import os
import sys
import commands
import string
import time
import fcntl
import getpass
#
from log_file import *
#
class PauseList:
    """ Paused Host List """
    def __init__(self,fileName):
        """ lock pause list and read it """
        self.pausedList = os.path.abspath(fileName) + ".pauseHostList"
        #
        # Check lock file
        self.lockFile = "/tmp/" + getpass.getuser()
        self.lockFile += os.path.basename(os.path.dirname(self.pausedList)) 
        self.lockFile += os.path.basename(self.pausedList) + ".lock"
        #
        #### LOCK THE LOCK FILE ####
        self.lockFileDescriptor = open(self.lockFile,"w+")
        fcntl.flock(self.lockFileDescriptor.fileno(),fcntl.LOCK_EX)        
        #
        # Read List
        if (True ==  os.path.isfile(self.pausedList)):
            inp = open(self.pausedList,'r')
            hosts = inp.readlines()
            inp.close()
            self.pHosts = []
            for host in hosts:
                ent = host.split()
                self.pHosts.append(ent[0])
        else:
            self.pHosts = []
        #
    def finalize(self):
        """ update List and unlock """
        #
        # update List
        outp = open(self.pausedList,'wt')        
        for host in self.pHosts:
            outp.write(host)
            outp.write("\n")
        outp.close()
        #
        ##### UNLOCK THE LOCK FILE ###
        fcntl.flock(self.lockFileDescriptor.fileno(),fcntl.LOCK_UN)
        self.lockFileDescriptor.close()
        #
    def initialize(self):
        """ initialize List  """
        self.pHosts = []
    #
    #
    def getPausedHost(self):
        """ returns paused cluster """

        if (len(self.pHosts) <= 0):
            return None
        
        if(self.pHosts[-1] == "MJ_SUSPENDED"):
            return None

        host = self.pHosts[-1]
        del self.pHosts[-1]
        #
        ans = host.split()
        return ans[0]
    #
    #
    def addPausedHost(self,cname):
        """ add host to pause """
        na = cname.split()
        self.pHosts.append(na[0])
    #
    #
    def countHost(self,cname):
        """ count host on list """
        na = cname.split()
        count = self.pHosts.count(na)
        #
        return count
    #
    #
    def suspendAllHost(self):
        """ Suspend All Host from New Submission """
        self.pHosts.append("MJ_SUSPENDED")    
#       
#
#
#
