#
#
# Status Database
#
# 05/31/2011 Hideki Fujioka
# 04/05/2012 Last Modified by Hideki Fujioka
#
import os
import sys
import commands
import string
import time
import fcntl
import getpass
import datetime
from xml.dom.minidom import getDOMImplementation
from xml.dom.minidom import parse
class StatusDatabase:
    """ Status Database """
    def __init__(self,fileName):
        """ lock Status database and read it """
        # fileName is ManyJobs Database file
        self.StatusDatabase = os.path.abspath(fileName) + ".xml"

        # get bindir
        ManyJobsDir = os.path.dirname(os.path.abspath(sys.argv[0])) 
        ManyJobsDir += 'localbin'

        #### LOCK THE LOCK FILE ####
        self.lockFile = "/tmp/" + getpass.getuser()
        self.lockFile += os.path.basename(os.path.dirname(fileName)) 
        self.lockFile += os.path.basename(fileName) + "_XML.lock"
        #
        self.lockFileDescriptor = open(self.lockFile,"w+")
        fcntl.flock(self.lockFileDescriptor.fileno(),fcntl.LOCK_EX)
        
        # Create DOM object
        impl = getDOMImplementation()
        self.dom = impl.createDocument(self.StatusDatabase,
                                        "StatusDatabase",
                                        None)        
        self.File = 0
        if (True ==  os.path.isfile(self.StatusDatabase)):
            self.dom = parse(self.StatusDatabase)
            self.File = 1
        else:
            st = 'type=\"text/xsl\" href=\"MJstatus.xslt\"'
            pi = self.dom.createProcessingInstruction('xml-stylesheet',st)
            doc = self.dom.documentElement
            self.dom.insertBefore(pi, doc)
        #    
        #doc = self.dom.documentElement
    #
    def finalize(self):
        """ update Status Database and unlock """
        #
        # update database
        fout = open(self.StatusDatabase, 'w')
        self.dom.writexml(fout)
        fout.close()

        #### UNLOCK THE LOCK FILE ###
        fcntl.flock(self.lockFileDescriptor.fileno(),fcntl.LOCK_UN)
        self.lockFileDescriptor.close()
        #
        #
    #
    #
    def addJobSubmitted(self,hName,jobid):
        """ Add Host job submitted  """
        doc = self.dom.documentElement
        job = self.dom.createElement('JOB')

        # add hostanme
        hostname = self.dom.createElement('hostname')
        hostname.appendChild(self.dom.createTextNode(hName))
        hostname.setAttribute('id',jobid)
        job.appendChild(hostname)
        
        # Add time submitted
        submitTime = self.dom.createElement('timeSubmitted')
        self.addTimeStamp(submitTime)
        job.appendChild(submitTime)

        # Add Status Queued
        status = self.dom.createElement('Status')
        status.appendChild(self.dom.createTextNode('Queued'))
        job.appendChild(status)
        
        # add to doc
        doc.appendChild(job)
    #
    #
    def reportJobToRunWithTasks(self,hName,jobid,numberOfTasks,taskNames,leadTasks):
        """ report Job to run with task """
        doc = self.dom.documentElement
        # Job Record
        jobs = doc.getElementsByTagName('JOB')
        for job in jobs:
            hostNameArray = job.getElementsByTagName('hostname')
            node = hostNameArray[0].childNodes
            att_node = hostNameArray[0].getAttributeNode('id')
            if (node[0].nodeValue == hName):
                if (att_node.nodeValue == jobid):
                    # add task names
                    for i in range(numberOfTasks):
                        task = self.dom.createElement('taskname')
                        task.appendChild(self.dom.createTextNode(taskNames[i]))
                        job.appendChild(task)
                    #    
                    # Add time task starts
                    startTime = self.dom.createElement('StartTime')
                    self.addTimeStamp(startTime)
                    job.appendChild(startTime)

                    # Status
                    status = job.getElementsByTagName('Status')
                    cnode = status[0].firstChild
                    status[0].removeChild(cnode)
                    status[0].appendChild(self.dom.createTextNode('Active'))
                    
                    break
                #
            #
                
        # Task Record
        for i in range(numberOfTasks):
            task = self.dom.createElement('TASK')                    
            # add task name
            tName = self.dom.createElement('name')
            tName.appendChild(self.dom.createTextNode(taskNames[i]))
            task.appendChild(tName)
            # Add time task starts
            startTime = self.dom.createElement('StartTime')
            self.addTimeStamp(startTime)
            task.appendChild(startTime)
            # Add host
            hostname = self.dom.createElement('hostname')
            hostname.appendChild(self.dom.createTextNode(hName))
            hostname.setAttribute('id',jobid)
            task.appendChild(hostname)
            # add leading task name
            ltName = self.dom.createElement('LeadingTask')
            ltName.appendChild(self.dom.createTextNode(leadTasks[i]))
            task.appendChild(ltName)
            #
            doc.appendChild(task)        
    #
    #
    def reportJobToFinished(self,hName,jobid):
        """ report Job to be finished """
        doc = self.dom.documentElement
        # Job Record
        jobs = doc.getElementsByTagName('JOB')
        for job in jobs:
            hostNameArray = job.getElementsByTagName('hostname')
            node = hostNameArray[0].childNodes
            att_node = hostNameArray[0].getAttributeNode('id')
            if (node[0].nodeValue == hName):
                if (att_node.nodeValue == jobid):
                    # Add time JOB ends
                    endTime = self.dom.createElement('EndTime')
                    self.addTimeStamp(endTime)
                    job.appendChild(endTime)

                    # Status
                    status = job.getElementsByTagName('Status')
                    cnode = status[0].firstChild
                    status[0].removeChild(cnode)
                    status[0].appendChild(self.dom.createTextNode('End'))
                    break
                #
            # 
        #
    #
    #
    def reportTasksToFinished(self,numOfTasks,taskNames,statuses,jobid):
        """ report Task to be finished """
        doc = self.dom.documentElement
        # Task Record
        tasks = doc.getElementsByTagName('TASK')
        for i in range(numOfTasks):
            for task in tasks:
                nameArray = task.getElementsByTagName('name')
                node = nameArray[0].childNodes
                hostArray = task.getElementsByTagName('hostname')
                jid = hostArray[0].getAttribute('id')
                if (node[0].nodeValue == taskNames[i]):
                    if (jid == jobid):
                        # Add time Task ends
                        endTime = self.dom.createElement('EndTime')
                        self.addTimeStamp(endTime)
                        task.appendChild(endTime)
                        # Status
                        stat = self.dom.createElement('TaskStatus')
                        stat.appendChild(self.dom.createTextNode(statuses[i]))
                        task.appendChild(stat)
                        break
                    #
                #
            #
        #
    #
    #
    #
    def getUnfinishedJobList(self):
        """ return list of queued/active jobs """
        doc = self.dom.documentElement
        jobs = doc.getElementsByTagName('JOB')
        host =[]
        jobID = []
        josStatus = []
        tasks = []
        for job in jobs:
            hostNameArray = job.getElementsByTagName('hostname')
            node = hostNameArray[0].childNodes
            att_node = hostNameArray[0].getAttributeNode('id')
            # Status
            status = job.getElementsByTagName('Status')
            cnode = status[0].firstChild
            if (cnode.nodeValue == 'Active') or (cnode.nodeValue == 'Queued'):
                host.append(node[0].nodeValue)
                jobID.append(att_node.nodeValue)
                josStatus.append(cnode.nodeValue)
                #

                tasknameArray = job.getElementsByTagName('taskname')
                tasklist=[]
                if (cnode.nodeValue == 'Active'):
                    for tasknamenode in tasknameArray:
                        tnode = tasknamenode.firstChild
                        tasklist.append(tnode.nodeValue)
                tasks.append(tasklist)
                #
        #
        return [host,jobID,josStatus,tasks]
    #
    #
    def getActiveJobList(self):
        """ return list of active jobs with start time"""
        doc = self.dom.documentElement
        jobs = doc.getElementsByTagName('JOB')
        host =[]
        jobID = []
        startTime = []
        tasks = []
        for job in jobs:
            hostNameArray = job.getElementsByTagName('hostname')
            node = hostNameArray[0].childNodes
            att_node = hostNameArray[0].getAttributeNode('id')
            # Status
            status = job.getElementsByTagName('Status')
            cnode = status[0].firstChild
            if (cnode.nodeValue == 'Active'):
                host.append(node[0].nodeValue)
                jobID.append(att_node.nodeValue)
                #
                sTime = job.getElementsByTagName('StartTime')
                startTime.append(self.getTimeStamp(sTime[0]))
                #
                tasknameArray = job.getElementsByTagName('taskname')
                tasklist=[]
                for tasknamenode in tasknameArray:
                    tnode = tasknamenode.firstChild
                    tasklist.append(tnode.nodeValue)
                #
                tasks.append(tasklist)
        #
        return [host,jobID,startTime,tasks]
    #
    #
    def getQueuedJobList(self):
        """ return list of queued jobs """
        doc = self.dom.documentElement
        jobs = doc.getElementsByTagName('JOB')
        host =[]
        jobID = []
        for job in jobs:
            hostNameArray = job.getElementsByTagName('hostname')
            node = hostNameArray[0].childNodes
            att_node = hostNameArray[0].getAttributeNode('id')
            # Status
            status = job.getElementsByTagName('Status')
            cnode = status[0].firstChild
            if (cnode.nodeValue == 'Queued'):
                host.append(node[0].nodeValue)
                jobID.append(att_node.nodeValue)
            #
        #
        return [host,jobID]
    #
    #
    def addTimeStamp(self,elm):
        """ add current time record """
        d = datetime.datetime.today()
        year = self.dom.createElement('year')
        year.appendChild(self.dom.createTextNode(str(d.year)))
        elm.appendChild(year)
        month = self.dom.createElement('month')
        month.appendChild(self.dom.createTextNode(str(d.month)))
        elm.appendChild(month)
        day = self.dom.createElement('day')
        day.appendChild(self.dom.createTextNode(str(d.day)))
        elm.appendChild(day)
        hour = self.dom.createElement('hour')
        hour.appendChild(self.dom.createTextNode(str(d.hour)))
        elm.appendChild(hour)
        minute = self.dom.createElement('minute')
        minute.appendChild(self.dom.createTextNode(str(d.minute)))
        elm.appendChild(minute)
        second = self.dom.createElement('second')
        second.appendChild(self.dom.createTextNode(d.strftime("%S")))
        elm.appendChild(second)
    #
    def getTimeStamp(self,timenode):
        """ get time recorded """
        yearA = timenode.getElementsByTagName('year')
        yn = yearA[0].firstChild
        year = yn.nodeValue
        #
        monthA = timenode.getElementsByTagName('month')
        mn = monthA[0].firstChild
        month = mn.nodeValue        
        #
        dayA = timenode.getElementsByTagName('day')
        dn = dayA[0].firstChild
        day = dn.nodeValue        
        #
        hourA = timenode.getElementsByTagName('hour')
        hn = hourA[0].firstChild
        hour = hn.nodeValue        
        #
        minuteA = timenode.getElementsByTagName('minute')
        mun = minuteA[0].firstChild
        minute = mun.nodeValue        
        #
        secondA = timenode.getElementsByTagName('second')
        sn = secondA[0].firstChild
        second = sn.nodeValue
        #
        d = datetime.datetime(year=int(year),
                              month=int(month),
                              day=int(day),
                              hour=int(hour),
                              minute=int(minute),
                              second=int(second)) 
        return d
#
#
