#!/usr/bin/env python
#
# Assign Task(s)
#
# input1 : Cluster Name
# input2 : jobID
# input3 : path to database on local machine
# input4 : Number of tasks to request
# input5 : Minimum number of tasks needed
# output: Environmental Variables
#
# 03/13/2012 Modified by Hideki Fujioka
#
import sys
import string
import os
import commands
import time
#
import ManyJobsManager
import PauseList
import StatusDatabase
from log_file import *
#
cName = sys.argv[1]
jobID = sys.argv[2]
manyJobsDB = sys.argv[3]
numberOfTasks = int(sys.argv[4])
minimumNumberOfTasks = int(sys.argv[5])
#
msg = "Request a Task from %s ID=%s" % (cName,jobID)
log_file(os.path.dirname(manyJobsDB),msg)
# init ManyJOBS manager
mjm = ManyJobsManager.ManyJobsManager(manyJobsDB)
# 
numberOfAvailableTasks = mjm.numberOfTasksToRun()
if (numberOfAvailableTasks < minimumNumberOfTasks):
    print None
    # finalize ManyJOBS manager
    mjm.finalize()
    sys.exit(-1)    
#
if (numberOfAvailableTasks < numberOfTasks):
    numberOfTasks = numberOfAvailableTasks
#
taskNameArray = []
jobFileArray = []
leadTasks = []
for n in range(numberOfTasks):
    # get a task to do
    [taskName, jobFile] = mjm.assignNewTask(cName,jobID)    
    taskNameArray.append(taskName)
    jobFileArray.append(jobFile)
    leadTask = mjm.getLeadingTask(taskName)
    leadTasks.append(leadTask)
# finalize ManyJOBS manager
mjm.finalize()
#
#
print "MANYJOBS_NUMBER_OF_TASKS=%s" % (numberOfTasks)
#
# read Variables
for n in range(numberOfTasks):
    taskName = taskNameArray[n]
    # send to computing node
    print "MANYJOBS_TASK_NAME=%s" % (taskNameArray[n])
    jobEnv = os.path.dirname(manyJobsDB) + '/' + jobFileArray[n]
    if (True ==  os.path.isfile(jobEnv)):
        inp = open(jobEnv)
        env = inp.readlines()
        inp.close()
        #
        for vari in env:
            print vari.split('\n')[0]
            #
    else:
        print "MANYJOBS_TASK_ENV=NOTFOUND"
    print "MANYJOBS_TASK_ENV=END"
    #
    # Append to LOG File
    #
    msg = "Start %s on %s ID=%s" % (taskName,cName,jobID)
    log_file(os.path.dirname(manyJobsDB),msg)
    #
    #
# Update Job Status
statusDB = StatusDatabase.StatusDatabase(manyJobsDB)
statusDB.reportJobToRunWithTasks(cName,jobID,numberOfTasks,taskNameArray,leadTasks)
statusDB.finalize()
#
#
# If no more available tasks at this point, kill all queued jobs and
# put cluster names on the paused list.
#
# init ManyJOBS manager
mjm = ManyJobsManager.ManyJobsManager(manyJobsDB)
# 
numberOfAvailableTasks = mjm.numberOfTasksToRun()
mjm.finalize()
#
msg = "There are %d tasks available" % (numberOfAvailableTasks)
log_file(os.path.dirname(manyJobsDB),msg)
if (numberOfAvailableTasks <= 0):
    statusDB = StatusDatabase.StatusDatabase(manyJobsDB)
    [Rname,RjobID] = statusDB.getQueuedJobList()
    statusDB.finalize()
    for i in range(len(Rname)):
        # kill jobs
        mjm = ManyJobsManager.ManyJobsManager(manyJobsDB)
        userssh = mjm.getSecureShell()
        rhost = mjm.getClusterAddressByName(Rname[i])
        jobLauch = mjm.getJobSubmitLauncherByName(Rname[i])
        maxJobs = mjm.getMaxJobsByName(Rname[i])
        ent = jobLauch.split(".py")
        jobKill = ent[0] + "_del.py"
        command = '%s %s %s %s' % (userssh,rhost, jobKill, RjobID[i])
        ans = commands.getoutput(command)
        #print command
        msg = "Kill " + RjobID[i] +" on " + Rname[i] + " since no task"
        log_file(os.path.dirname(manyJobsDB),msg)
        mjm.finalize()
        # put into pause list
        pList = PauseList.PauseList(manyJobsDB)
        if (maxJobs > pList.countHost(Rname[i])):
            pList.addPausedHost(Rname[i])
        pList.finalize()        
        # update database
        statusDB = StatusDatabase.StatusDatabase(manyJobsDB)            
        statusDB.reportJobToFinished(Rname[i],RjobID[i])
        statusDB.finalize()
    #
#
#
