#!/usr/bin/env python
##\file job_loni.py
##\brief Submit a Job to LONI LINUX Cluster
##\author Hideki Fujioka
##\date 11/11/2009
##\return \ref MANYJOBS_JOBID
##
## 05/09/2011 Hidek Fujioka : last modified
##
"""
\page JobSubmisionScripts Job Submission Scripts

\section intro_JobSubmisionScripts Introduction

This section will explain how to create job-submission scripts. 
The job-submission script is executed remotely using ssh/gsissh. 
This script submits a job onto NQS system and returns the job-ID.
You have to write this script specifically for each cluster and each purpose.

The job-submission script executes a job submission command for NQS.
There are several things you need to decide and state in the script.

\li How many processors to request; 
the number of nodes and the number of processors within a node. 
These number is depended on the system and tasks you are going to
run on the system.

\li How long the job runs. 
You need to define a walltime of you jobs. 
The actual run-time of your tasks must be shorter than 
the walltime you set here. 
If your task runs until reaching the walltime, 
the job will be killed b NQS and ManyJobs won't 
know the termination of the task. 
Therefore, you should set the walltime to 
be the actual run-time of your longest task plus the pre and post process times.

\li Queue name.
There must be some choice of queue in the NQS.

\li Allocation Name.
The most of public cluster requires allocations to run jobs.

\li Other settings.
Any others you want to send to NQS.

\section inputs_reture_JobSubmisionScripts Input and Return values 
This script needs inputs: 
\ref NAMYJOBS_CLUSTER_NAME,
\ref MANYJOBS_MASTER_HOST,
\ref MANYJOBS_MASTER_DIR,
and
\ref MANYJOBS_MASTER_JOBS_DB.

The return value is a \ref MANYJOBS_JOBID.

\n
-------------------------------------------------------------------------

\page LONI_JS_script  Job Submission Script for LONI linux clusters
job_loni.py
[\ref NAMYJOBS_CLUSTER_NAME]
[\ref MANYJOBS_MASTER_HOST]
[\ref MANYJOBS_MASTER_DIR]
[\ref MANYJOBS_MASTER_JOBS_DB]

\param [in] NAMYJOBS_CLUSTER_NAME \ref NAMYJOBS_CLUSTER_NAME
\param [in] MANYJOBS_MASTER_HOST  \ref MANYJOBS_MASTER_HOST 
\param [in] MANYJOBS_MASTER_DIR \ref MANYJOBS_MASTER_DIR
\param [in] MANYJOBS_MASTER_JOBS_DB \ref MANYJOBS_MASTER_JOBS_DB

\return \ref MANYJOBS_JOBID

Parameters
\li Allocation Name
\code
allocationName = "loni_acinus02" 
\endcode
\li Wall Time             
\code
wallTime = "1:00:00"
\endcode
\li Number of Processors
\code
nproc = "nodes=1:ppn=4" 
\endcode
\li Queue Name
\code
queueName = "workq"              
\endcode
\li Email
\code
email = "hidekin2549@hotmail.com"
\endcode

SETTING NUMBER OF SUBJOBS
\li \ref MANYJOBS_NUMOFSUBJOBS -> MANYJOBS_NUMOFSUBJOBS
\li \ref MANYJOBS_MINNUMOFSUBJOBS -> MANYJOBS_MINNUMOFSUBJOBS
\code
os.putenv('MANYJOBS_NUMOFSUBJOBS','1')
os.putenv('MANYJOBS_MINNUMOFSUBJOBS','1')
\endcode

Job Submission Command:
\code
command = "qsub -N %s " % (jobName) 
command += "-A %s " % (allocationName)
command += "-l walltime=%s " % (wallTime)
command += "-l %s " % (nproc)
command += "-q %s " % (queueName)
command += "-M %s " % (email)
command += "-m abe -k oe -V "
command += "%s" % (job)
\endcode
Note that option '-V ' is important!! \n



"""
import sys
import string
import os
import commands
import time
import subprocess
#
dirname = os.path.dirname(sys.argv[0])
#
remoteBinDir = os.path.dirname(dirname) + "/remotebin"
sys.path.append(remoteBinDir)
from log_file import *
#
logDir =  os.path.dirname(remoteBinDir) + "/log"
# 
##\name Information  for PBS (qsub) 
##@{
## parameters

## Allocation Name
allocationName = "loni_acinus02" 
## Wall Time             
wallTime = "1:00:00"
## Number of Processors
nproc = "nodes=1:ppn=4" 
## Job Name
jobName = "MJ-" + os.path.basename(sys.argv[4])
## Queue Name
queueName = "workq"              
## Email
email = "hidekin2549@hotmail.com"
## Job Script (Do Not Change)
job = remoteBinDir + "/job_launcher.sh" 
##
## SETTING NUMBER OF SUBJOBS
## \ref MANYJOBS_NUMOFSUBJOBS
## \ref MANYJOBS_MINNUMOFSUBJOBS
## \code
## os.putenv('MANYJOBS_NUMOFSUBJOBS','1')
## os.putenv('MANYJOBS_MINNUMOFSUBJOBS','1')
## \endcode
os.putenv('MANYJOBS_NUMOFSUBJOBS','1')
os.putenv('MANYJOBS_MINNUMOFSUBJOBS','1')
##@}
##
##\name Job Submission Command
##@{
command = "qsub -N %s " % (jobName) 
command += "-A %s " % (allocationName)
command += "-l walltime=%s " % (wallTime)
command += "-l %s " % (nproc)
command += "-q %s " % (queueName)
command += "-M %s " % (email)
command += "-m abe -k oe -V "
command += "%s" % (job)
## \code
##command = "qsub -N %s " % (jobName) 
##command += "-A %s " % (allocationName)
##command += "-l walltime=%s " % (wallTime)
##command += "-l %s " % (nproc)
##command += "-q %s " % (queueName)
##command += "-M %s " % (email)
##command += "-m abe -k oe -V "
##command += "%s" % (job)
##\endcode
## option '-V ' is important!!
##@}
#
## log
log_file(logDir,command)
#
## set evironment variables
## \ref MANYJOBS_CLUSTER_NAME
os.putenv('MANYJOBS_CLUSTER_NAME', sys.argv[1])
## \ref MANYJOBS_MASTER_HOST
os.putenv('MANYJOBS_MASTER_HOST', sys.argv[2])
## \ref MANYJOBS_MASTER_DIR
os.putenv('MANYJOBS_MASTER_DIR', sys.argv[3])
## \ref MANYJOBS_MASTER_JOBS_DB
os.putenv('MANYJOBS_MASTER_JOBS_DB', sys.argv[4])
## \ref MANYJOBS_SCRIPT_DIR
os.putenv('MANYJOBS_SCRIPT_DIR', remoteBinDir)
## \ref MANYJOBS_LOG_DIR
os.putenv('MANYJOBS_LOG_DIR', logDir)
## \ref MANYJOBS_MPI
os.putenv('MANYJOBS_MPI', 'mvapich2')
## \ref MANYJOBS_BATCH
os.putenv('MANYJOBS_BATCH', 'pbs')
## \ref MANYJOBS_SCRATCHDIR
os.putenv('MANYJOBS_SCRATCHDIR', '/work/fuji/')
## \ref MANYJOBS_SSHCOMMAND
os.putenv('MANYJOBS_SSHCOMMAND', 'ssh') # ssh command works on computing nodes
#
## \ref MANYJOBS_HEADNODE
HName = commands.getoutput("hostname")
os.putenv('MANYJOBS_HEADNODE', HName)
#
#
## SUBMIT A JOB TO PBS
os.chdir(logDir)
ans = subprocess.Popen(command,
                       shell=True,
                       stdout=subprocess.
                       PIPE,stderr=subprocess.PIPE)
stdval = ans.communicate()[0]
print "************\n"
print "MANYJOBID=%s\n" % (stdval.replace('\n',''))
print "************\n"
#
#
# log
log_file(logDir,stdval)
