#!/usr/bin/env python
##\file job_loni2.py
##\brief Submit a Job to LONI LINUX Cluster
##\author Hideki Fujioka
##\date 01/09/2012
##\return \ref MANYJOBS_JOBID
##
## 01/09/2012 Hidek Fujioka : last modified
##
"""
\page JobSubmisionScripts Job Submission Scripts

\section intro_JobSubmisionScripts Introduction

This section will explain how to create job-submission scripts. 
The job-submission script is executed remotely using ssh/gsissh. 
This script submits a job onto NQS system and returns the job-ID.
You have to write this script specifically for each cluster and each purpose.

The job-submission script executes a job submission command for NQS.
There are several things you need to decide and state in the script.

This example script get option from DB.

\section inputs_reture_JobSubmisionScripts Input and Return values 
This script needs inputs: 
\ref NAMYJOBS_CLUSTER_NAME,
\ref MANYJOBS_MASTER_HOST,
\ref MANYJOBS_MASTER_DIR,
\ref MANYJOBS_MASTER_JOBS_DB,
and
\ref optionString

The return value is a \ref MANYJOBS_JOBID.

\n
-------------------------------------------------------------------------

\page LONI_JS_script2  Job Submission Script for LONI linux clusters 2
job_loni2.py
[\ref NAMYJOBS_CLUSTER_NAME]
[\ref MANYJOBS_MASTER_HOST]
[\ref MANYJOBS_MASTER_DIR]
[\ref MANYJOBS_MASTER_JOBS_DB]
[\ref optionString]

\param [in] NAMYJOBS_CLUSTER_NAME \ref NAMYJOBS_CLUSTER_NAME
\param [in] MANYJOBS_MASTER_HOST  \ref MANYJOBS_MASTER_HOST 
\param [in] MANYJOBS_MASTER_DIR \ref MANYJOBS_MASTER_DIR
\param [in] MANYJOBS_MASTER_JOBS_DB \ref MANYJOBS_MASTER_JOBS_DB
\param [in] optionString \ref optionString

\return \ref MANYJOBS_JOBID


SETTING NUMBER OF SUBJOBS
\li \ref MANYJOBS_NUMOFSUBJOBS -> MANYJOBS_NUMOFSUBJOBS
\li \ref MANYJOBS_MINNUMOFSUBJOBS -> MANYJOBS_MINNUMOFSUBJOBS
\code
os.putenv('MANYJOBS_NUMOFSUBJOBS','1')
os.putenv('MANYJOBS_MINNUMOFSUBJOBS','1')
\endcode

Job Submission Command:
\code
command = "qsub -N %s " % (jobName) 
command += pbsOptions
command += "-M %s " % (email)
command += "-m abe -k oe -V "
command += "%s" % (job)
\endcode
Note that option '-V ' is important!! \n


"""
import sys
import string
import os
import commands
import time
import subprocess
#
dirname = os.path.dirname(sys.argv[0])
#
remoteBinDir = os.path.dirname(dirname) + "/remotebin"
sys.path.append(remoteBinDir)
from log_file import *
#
logDir =  os.path.dirname(remoteBinDir) + "/log"
# 
##\name Information  for PBS (qsub) 
##@{
## parameters

# batch job information
optionString = sys.argv[5]
pbsOptions = optionString.replace("%20"," ")
#
## Job Name
jobName = "MJ-" + os.path.basename(sys.argv[4])
#
## Email
email = "email@hotmail.com"
#
## Job Script (Do Not Change)
job = remoteBinDir + "/job_launcher.sh" 
##
## SETTING NUMBER OF SUBJOBS
os.putenv('MANYJOBS_NUMOFSUBJOBS','1')
os.putenv('MANYJOBS_MINNUMOFSUBJOBS','1')
##@}
##
##\name Job Submission Command
##@{
command = "qsub -N %s " % (jobName)
command += pbsOptions
command += "-M %s " % (email)
command += "-m abe -k oe -V "
command += "%s" % (job)
#
## log
log_file(logDir,command)
#
## set evironment variables
os.putenv('MANYJOBS_CLUSTER_NAME', sys.argv[1])
os.putenv('MANYJOBS_MASTER_HOST', sys.argv[2])
os.putenv('MANYJOBS_MASTER_DIR', sys.argv[3])
os.putenv('MANYJOBS_MASTER_JOBS_DB', sys.argv[4])
os.putenv('MANYJOBS_SCRIPT_DIR', remoteBinDir)
os.putenv('MANYJOBS_LOG_DIR', logDir)
os.putenv('MANYJOBS_MPI', 'mvapich2')
os.putenv('MANYJOBS_BATCH', 'pbs')
os.putenv('MANYJOBS_SCRATCHDIR', '/work/userid/')
os.putenv('MANYJOBS_SSHCOMMAND', 'ssh') # ssh command works on computing nodes
#
## \ref MANYJOBS_HEADNODE
HName = commands.getoutput("hostname")
os.putenv('MANYJOBS_HEADNODE', HName)
#
#
## SUBMIT A JOB TO PBS
os.chdir(logDir)
ans = subprocess.Popen(command,
                       shell=True,
                       stdout=subprocess.
                       PIPE,stderr=subprocess.PIPE)
stdval = ans.communicate()[0]
print "************\n"
print "MANYJOBID=%s\n" % (stdval.replace('\n',''))
print "************\n"
#
#
# log
log_file(logDir,stdval)
